// FILIP STEVANOVIC - PRVI ZADATAK DRUGOG POPRAVNOG KOLOKVIJUMA

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <mpi.h>

#define PAPIR 0
#define KAMEN 1
#define MAKAZE 2

typedef struct st
{
	int oruzje;
	int brojPobeda;
	int brojPoraza;
} Stat;

typedef struct igrac
{
	int oruzje;
	int brojPobeda;
	int brojPoraza;
	int odigrao; // Ubacio kao indikator da je igrac odigrao
} IGRAC;

void SumStat(Stat *in, Stat *inout, int *len, MPI_Datatype *type)
{
	int i;
	
	for (i = 0; i < *len; i++)
	{
		if (inout->oruzje == in->oruzje)
		{
			inout->brojPobeda += in->brojPobeda;
			inout->brojPoraza += in->brojPoraza;
		}
		
		in++;
		inout++;
	}	
}

void MojMax(Stat *in, Stat *inout, int *len, MPI_Datatype *type)
{
	int i;
	
	for (i = 0; i < *len; i++)
	{
		if (inout->brojPobeda < in->brojPobeda)
		{
			inout->oruzje = in->oruzje;
			inout->brojPobeda = in->brojPobeda;
			inout->brojPoraza = in->brojPoraza;
		}
		
		in++;
		inout++;
	}	
}

IGRAC **allocateGhost(int m, int n)
{
	IGRAC **pom;
	int i;
	
	pom = (IGRAC **)malloc((m + 2) * sizeof(IGRAC *)); // Dodatne dve vrste za ghost razmenu
	
	for (i = 0; i < m + 2; i++)
	{
		pom[i] = (IGRAC *)malloc(n * sizeof(IGRAC));
	}
	
	return pom;
}

void odigraj(IGRAC *i1, IGRAC *i2)
{
	if (i1->oruzje == KAMEN)
	{
		if (i2->oruzje == PAPIR)
		{
			i1->brojPoraza += 1;
			i2->brojPobeda += 1;
		}
		else if (i2->oruzje == MAKAZE)
		{
			i1->brojPobeda += 1;
			i2->brojPoraza += 1;
		}
	}
	else if (i1->oruzje == PAPIR)
	{
		if (i2->oruzje == KAMEN)
		{
			i1->brojPobeda += 1;
			i2->brojPoraza += 1;
		}
		else if (i2->oruzje == MAKAZE)
		{
			i1->brojPoraza += 1;
			i2->brojPobeda += 1;
		}
	}
	else if (i1->oruzje == MAKAZE)
	{
		if (i2->oruzje == KAMEN)
		{
			i1->brojPoraza += 1;
			i2->brojPobeda += 1;
		}
		else if (i2->oruzje == PAPIR)
		{
			i1->brojPobeda += 1;
			i2->brojPoraza += 1;
		}
	}
	
	i1->odigrao = 1;
	i2->odigrao = 1;
}

int mozeDaOdigra(int i, int j, int n, int donjaGranica, int gornjaGranica) // Donja i gornja granica su maksimalni indeksi
{
	if (i < donjaGranica || j < 0 || i > gornjaGranica || j > n - 1)
	{
		return 0;
	}
	
	return 1;
}

int main(int argc, char **argv)
{
	int id, np, m, n, poKolikoVrsta, i, j, k, donjaGranica, gornjaGranica, brojacKrugova = 1;
	IGRAC **matrica, statistika[3];
	IGRAC konacno;
	MPI_Status status;
	
	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &id);
	MPI_Comm_size(MPI_COMM_WORLD, &np);
	
	int blockcounts[1];
	MPI_Datatype strukturaStat, strukturaIgrac, oldtypes[1];
	MPI_Aint offsets[1];
	MPI_Op myOp, myOp1;
	MPI_Op_create((MPI_User_function *) SumStat, 0, &myOp);
	MPI_Op_create((MPI_User_function *) MojMax, 0, &myOp1); // Funkcija za maksimum
	oldtypes[0] = MPI_INT;
	
	// Struktura za statistiku
	blockcounts[0] = 3;
	offsets[0] = 0;
	MPI_Type_struct(1, blockcounts, offsets, oldtypes, &strukturaStat);
	MPI_Type_commit(&strukturaStat);

	// Struktura za igraca
	blockcounts[0] = 4;
	offsets[0] = 0;
	MPI_Type_struct(1, blockcounts, offsets, oldtypes, &strukturaIgrac);
	MPI_Type_commit(&strukturaIgrac);
	
	srand(time(NULL) * (id + 10));
	
	if (id == 0)
	{
		printf("Unesite m, n i k:\n");
		
		scanf("%d%d%d", &m, &n, &k);
		
		poKolikoVrsta = m / np; // Broj vrsta po procesu
		
		for (i = 1; i < np; i++)
		{
			MPI_Send(&poKolikoVrsta, 1, MPI_INT, i, 10, MPI_COMM_WORLD);
			MPI_Send(&n, 1, MPI_INT, i, 15, MPI_COMM_WORLD);
			MPI_Send(&k, 1, MPI_INT, i, 20, MPI_COMM_WORLD);
		}
	}
	else
	{
		MPI_Recv(&poKolikoVrsta, 1, MPI_INT, 0, 10, MPI_COMM_WORLD, &status);
		MPI_Recv(&n, 1, MPI_INT, 0, 15, MPI_COMM_WORLD, &status);
		MPI_Recv(&k, 1, MPI_INT, 0, 20, MPI_COMM_WORLD, &status);
	}
	
	matrica = allocateGhost(poKolikoVrsta, n);
	
	for (i = 1; i < poKolikoVrsta + 1; i++)
	{	
		for (j = 0; j < n; j++)
		{
			matrica[i][j].oruzje = 3.0 * rand() / RAND_MAX;
			matrica[i][j].brojPobeda = 0;
			matrica[i][j].brojPoraza = 0;
			matrica[i][j].odigrao = 0;
		}
	}
	
	// Glavna petlja
	
	while (brojacKrugova <= k) // Dok ima krugova
	{
		// Razmena ghost vrsta
		
		if (id == 0)
		{
			MPI_Send(matrica[poKolikoVrsta], n, strukturaIgrac, id + 1, 10, MPI_COMM_WORLD);
			MPI_Recv(matrica[poKolikoVrsta + 1], n, strukturaIgrac, id + 1, 10, MPI_COMM_WORLD, &status);
		}
		else if (id == np - 1)
		{
			MPI_Recv(matrica[0], n, strukturaIgrac, id - 1, 10, MPI_COMM_WORLD, &status);
			MPI_Send(matrica[1], n, strukturaIgrac, id - 1, 10, MPI_COMM_WORLD);
		}
		else
		{
			MPI_Recv(matrica[0], n, strukturaIgrac, id - 1, 10, MPI_COMM_WORLD, &status);
			MPI_Send(matrica[1], n, strukturaIgrac, id - 1, 10, MPI_COMM_WORLD);
			
			MPI_Send(matrica[poKolikoVrsta], n, strukturaIgrac, id + 1, 10, MPI_COMM_WORLD);
			MPI_Recv(matrica[poKolikoVrsta + 1], n, strukturaIgrac, id + 1, 10, MPI_COMM_WORLD, &status);
		}
		
		// Odredjivanje gornjih i donjih granica (zbog prazne 0. vrste u procesu ciji je ID = 0 i prazne poslednje vrste u procesu ciji je ID = np - 1)
		
		if (id == 0)
		{
			donjaGranica = 1;
			gornjaGranica = poKolikoVrsta + 1;
		}
		else if (id == np - 1)
		{
			donjaGranica = 0;
			gornjaGranica = poKolikoVrsta;
		}
		else
		{
			donjaGranica = 0;
			gornjaGranica = poKolikoVrsta + 1;
		}
		
		// Glavni deo
		
		for (i = 0; i < poKolikoVrsta + 2; i++)
		{
			for (j = 0; j < n; j++)
			{
				if (mozeDaOdigra(i - 1, j - 1, n, donjaGranica, gornjaGranica))
				{
					if (matrica[i - 1][j - 1].odigrao == 0)
					{
						odigraj(&matrica[i][j], &matrica[i - 1][j - 1]);
					}
				}
				
				if (mozeDaOdigra(i - 1, j, n, donjaGranica, gornjaGranica))
				{
					if (matrica[i - 1][j].odigrao == 0)
					{
						odigraj(&matrica[i][j], &matrica[i - 1][j]);
					}
				}
				
				if (mozeDaOdigra(i - 1, j + 1, n, donjaGranica, gornjaGranica))
				{
					if (matrica[i - 1][j + 1].odigrao == 0)
					{
						odigraj(&matrica[i][j], &matrica[i - 1][j + 1]);
					}
				}
				
				if (mozeDaOdigra(i, j - 1, n, donjaGranica, gornjaGranica))
				{
					if (matrica[i][j - 1].odigrao == 0)
					{
						odigraj(&matrica[i][j], &matrica[i][j - 1]);
					}
				}
				
				if (mozeDaOdigra(i, j + 1, n, donjaGranica, gornjaGranica))
				{
					if (matrica[i][j + 1].odigrao == 0)
					{
						odigraj(&matrica[i][j], &matrica[i][j + 1]);
					}
				}
				
				if (mozeDaOdigra(i + 1, j - 1, n, donjaGranica, gornjaGranica))
				{
					if (matrica[i + 1][j - 1].odigrao == 0)
					{
						odigraj(&matrica[i][j], &matrica[i + 1][j - 1]);
					}
				}
				
				if (mozeDaOdigra(i + 1, j, n, donjaGranica, gornjaGranica))
				{
					if (matrica[i + 1][j].odigrao == 0)
					{
						odigraj(&matrica[i][j], &matrica[i + 1][j]);
					}
				}
				
				if (mozeDaOdigra(i + 1, j + 1, n, donjaGranica, gornjaGranica))
				{
					if (matrica[i + 1][j + 1].odigrao == 0)
					{
						odigraj(&matrica[i][j], &matrica[i + 1][j + 1]);
					}
				}
			}
		}
		
		// Odabir za novi krug
		
		for (i = 1; i < poKolikoVrsta + 1; i++)
		{	
			for (j = 0; j < n; j++)
			{
				matrica[i][j].oruzje = 3.0 * rand() / RAND_MAX;
				matrica[i][j].odigrao = 0;
			}
		}
		
		// Sumiranje statistike
		
		MPI_Reduce(matrica, statistika, 3, strukturaStat, myOp, 0, MPI_COMM_WORLD);
		
		brojacKrugova++;
		
		
		
		MPI_Barrier(MPI_COMM_WORLD);
	}
	
	// Najbolji igrac
	
	MPI_Reduce(matrica, &konacno, 1, strukturaIgrac, myOp1, 0, MPI_COMM_WORLD);
	
	printf("Broj pobeda najboljeg igraca: %d\n", konacno.brojPobeda);
	
	MPI_Finalize();
	
	return 0;
}
