#include <boost/mpi/environment.hpp>
#include <boost/mpi/communicator.hpp>
#include <boost/mpi.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/string.hpp>
#include <iostream>
#include <vector>
#include <sstream>
#include <boost/array.hpp>
#include <boost/mpi/collectives.hpp>
#include <stdio.h>
#include <stdlib.h>

namespace mpi = boost::mpi;
using namespace std;

class Stat
{
public:
	int id;
	int br_poena;
	int br_skokova;
	int br_asistencija;
public:
	Stat(){}
	~Stat(){}
	Stat(int i,int p, int s, int a)
	{
		this->id = id;
		this->br_poena = p;
		this->br_skokova = s;
		this->br_asistencija = a;
	}
	
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & id;
		ar & br_poena;
		ar & br_skokova;
		ar & br_asistencija;
	}
};

class Kosarkas
{
public:
	int id;
	string ime;
	int br_poena;
	int br_asistencija;
	int br_skokova;
	int br_utakmica;
public:
	Kosarkas(){}
	~Kosarkas(){}
	Kosarkas(int i, string name)
	{
		this->id = i;
		strcpy(this->ime, name);
		this->br_poena = 0;
		this->br_asistencija = 0;
		this->br_skokova = 0;
		this->br_utakmica = 0;
	}
	
	void AddStat(Stat stats)
	{
		this->br_poena += stats.br_poena;
		this->br_asistencija += stats.br_asistencija;
		this->br_skokova += stats.br_skokova;
		this->br_utakmica++;
	}
	
	bool IsPlayer(int i)
	{
		bool result = false;
		if (this->id == i)
		{
			result = true;
		}
		return result;
	}
	
	//true ako je poslati bolji od trenutnog
	bool Operator(Kosarkas player)
	{
		bool result = false;
		double mvp1, mvp2;
		
		mvp1 = ((double) (this->br_poena*2 + this->br_asistencija*3 + this->br_skokova*3)/this->br_utakmica);
		mvp2 = ((double) (player.br_poena*2 + player.br_asistencija*3 + player.br_skokova*3)/player.br_utakmica);
		
		if (mvp1 < mvp2)
		{
			result = true;
		}
		return result;
	}
	
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & id;
		ar & br_poena;
		ar & br_skokova;
		ar & br_asistencija;
		ar & ime;
		ar & br_utakmica;
	}
}

int main(int argc, char* argv[]) 
{
  mpi::environment env(argc, argv);
  mpi::communicator world;
  
  int i,j;
  
  vector<Kosarkas> kosarkasi;
  vector<Stat> statistika;
  int poKoliko,brStats;
  Kosarkas *mojiKosarkasi;
  
  if (world.rank() == 0)
  {
	  FILE* f = fopen("kosarkasi.dat","r");
	  int brKosarkasa;
	  
	  int id;
	  string ime;
	  Kosarkas k;
	  
	  fscanf(f,"%d\n",&brKosarkasa);
	  for (int i = 0; i < brKosarkasa; i++)
	  {
		  fscanf(f,"%d\n",&id);
		  fscanf(f,"%s\n",ime);
		  k = Kosarkas(id,ime);
		  kosarkasi.push_back(k);
	  }
	  
	  fclose(f);
	  
	  poKoliko = brKosarkasa/world.size();
  }
  
  broadcast(world,poKoliko,0);
  mojiKosarkasi = new Kosarkas[poKoliko];
  
  scatter(world,kosarkasi,mojiKosarkasi,poKoliko,0);
  
  if (world.rank() == 0)
  {
	  FILE* f = fopen("stats.dat","r");
	  
	  int id, br_poena, br_asistencija, br_skokova;
	  Stat s;
	  
	  fscanf(f,"%d\n",&brStats);
	  for (int i = 0; i < brStats; i++)
	  {
		  fscanf(f,"%d %d %d %d\n",id,br_poena,br_skokova,br_asistencija);
		  s = Stat(id,br_poena,br_skokova,br_asistencija);
		  statistika.push_back(s);
	  }
	  fclose(f);
  }
  
  broadcast(world,statistika,0);
  broadcast(world,brStats,0);
  
  for (int i = 0; i < poKoliko; i++)
  {
	  for (int j = 0; j < brStats; j++)
	  {
		  if (mojiKosarkasi[i].IsPlayer(statistika[j].id))
		  {
			  mojiKosarkasi[i].AddStat(statistika[j]);
		  }
	  }
  }
  
  Kosarkas pom = mojiKosarkasi[0];
  for (int i = 1; i < poKoliko; i++)
  {
	  if (pom.Operator(mojiKosarkasi[i]))
	  {
		  pom = mojiKosarkasi[i];
	  }
  }
  
  Kosarkas najNiz = new Kosarkas[world.size()];
  gather(world, najNiz, pom, 0);//skupi sve najbolje
  if (world.rank() == 0)
  {
	  for (int i = 0; i < world.size(); i++)
	  {
		  if (pom.Operator(najNiz[i]))
		  {
			  pom = najNiz[i];
		  }
	  }
	  printf("Najbolji kosarkas se zove %s i njegov id je %d\n",pom.ime,pom.id);
  }
  
  
  return 0;
}
