#include <boost/mpi/environment.hpp>
#include <boost/mpi/communicator.hpp>
#include <boost/mpi.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/string.hpp>
#include <iostream>
#include <vector>
#include <sstream>
#include <boost/array.hpp>
#include <boost/mpi/collectives.hpp>
#include <stdio.h>
#include <string.h>

namespace mpi = boost::mpi;
using namespace std;
  
class Stat{
	public:
		friend class boost::serialization::access;
		template<class Archive>
		void serialize(Archive & ar, const unsigned int version)
		{
			ar & id;
			ar & brPoena;
			ar & brSkokova;
			ar & brAsistencija;
		}
		
		int id;
		int brPoena;
		int brSkokova;
		int brAsistencija;
		
	public:
		Stat(){}
		Stat(int a,int b,int c,int d){
			id = a;
			brPoena = b;
			brSkokova = c;
			brAsistencija = d;
		}
};

class Kosarkas{
	public:
		friend class boost::serialization::access;
		template<class Archive>
		void serialize(Archive & ar, const unsigned int version)
		{
			ar & id;
			ar & brPoena;
			ar & brSkokova;
			ar & brAsistencija;
			ar & brUtakmica;
			ar & ime;
		}
		int id;
		int brPoena;
		int brSkokova;
		int brAsistencija;
		int brUtakmica;
		char ime[20]; //imao sam problema sa tipom string 
	
	public:
		Kosarkas();
		Kosarkas(int i, char s[]){
			id = i;
			strcpy(ime,s);
			brUtakmica = 0;
			brPoena = 0;
			brSkokova = 0;
			brAsistencija = 0;
		}
		
		void AddStat(int poeni, int asistencije, int skokovi){
			brPoena += poeni;
			brSkokova += skokovi;
			brAsistencija += asistencije;
			brUtakmica++;
		}
		
		bool IsPlayer(int i){
			if(id == i){
				return true;
			}
			return false;
		}
		
		bool operator > (Kosarkas k){
			if(mvp_rate() > k.mvp_rate())
				return true;
			return false;
		}
		
		float mvp_rate(){
			if(brUtakmica==0)return 0;
			return (brPoena*2+brAsistencija*3+brSkokova*3)/brUtakmica;
		}		
};


main(int argc, char **argv){
	
	system("clear");
	
	mpi::environment env(argc, argv);
	mpi::communicator world;
	
	int id = world.rank();
	int p = world.size();
	
	vector<Stat> statistika;
	vector<Kosarkas> kosarkasi;
	
	int i,j,n;
	
	if(id==0){	
		FILE *f = fopen("kosarkasi","r");
		if(f!=NULL){	
			int id;
			char s[20];
			while(1){
				fscanf(f,"%d%s",&id,&s);
				if(feof(f))break;
				Kosarkas k(id,s);
				kosarkasi.push_back(k);			
			}		
		}else
			printf("error opening file kosarkasi...\n");
			
		f = fopen("statistika","r");
		if(f!=NULL){
			int id,poeni,skokovi,asistencije;
			while(1){
				fscanf(f,"%d%d%d%d",&id,&poeni,&asistencije,&skokovi);
				if(feof(f))break;
				Stat s(id,poeni,skokovi,asistencije);
				statistika.push_back(s);			
			}		
		}else
			printf("error opening file statistika...\n");
		n = kosarkasi.size();
	}
	
	broadcast(world,statistika,0);
	broadcast(world,n,0);
	
	int div = n/p;
	
	Kosarkas *k = (Kosarkas*)malloc(div*sizeof(Kosarkas));
	scatter(world,kosarkasi,k,div,0);

	for(i=0;i<div;i++){
		for(j=0;j<statistika.size();j++){
			if(statistika[j].id == k[i].id){
				k[i].AddStat(statistika[j].brPoena,statistika[j].brAsistencija,statistika[j].brSkokova);
			}
		}	
	}
	
	
	Kosarkas max = k[0];
	for(i=1;i<div;i++){
		if(k[i]>max)
			max = k[i];
	}
	
	Kosarkas *mvp = (Kosarkas*)malloc(p*sizeof(Kosarkas));

	gather(world,max,mvp,0);
		
	if(id==0){
		max = mvp[0];
		for(i=1;i<p;i++)
			if(mvp[i]>max)
				max = mvp[i];
		printf("mvp: %s(%d)\nbroj poena: %d\nbroj asistencija: %d\nbroj skokova: %d\nbroj utakmica: %d\n",
				max.ime,max.id,max.brPoena,max.brAsistencija,max.brSkokova,max.brUtakmica);
	}
	return 0;
}



































