#include <boost/mpi/environment.hpp>
#include <boost/mpi/communicator.hpp>
#include <boost/mpi.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/string.hpp>
#include <iostream>
#include <vector>
#include <sstream>
#include <boost/array.hpp>
#include <boost/mpi/collectives.hpp>

namespace mpi = boost::mpi;
using namespace std;

class Stat
{
	
public:
	int id;	
	int br_poena;
	int br_skokova;
	int br_asistencija;
	
	
public:
  	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{	

	  ar & id;
	  ar & br_poena;
	  ar & br_skokova;
	  ar & br_asistencija;
	}

	
public:
	Stat(){}
	Stat(int _id, int _br_poena, int _br_skokova, int _br_asistencija)
	{ 
		id=_id;
		br_poena=_br_poena;
		br_skokova=_br_skokova;
		br_asistencija=_br_asistencija;
		
	}
	~Stat(){}


};


class Kosarkas
{
	
public:
	int id;	
	char ime[10];
	int br_poena;
	int br_asistencija;
	int br_skokova;
	int br_utakmica;
	
	
public:
  	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{	

	  ar & id;
	  ar & br_poena;
	  ar & br_skokova;
	  ar & br_asistencija;
	  ar & ime;
	  ar & br_utakmica;
	}

	
public:
	Kosarkas(){}
	Kosarkas(int _id, string imee)
	{ 
		id=_id;
		strcpy(ime,imee.c_str());
		br_utakmica=0;
		br_poena=0;
		br_asistencija=0;
		br_skokova=0;
		br_utakmica=0;
		
	}
	~Kosarkas(){}

	void AddStat(Stat s)
	{
		if(id==s.id)
		{
			br_poena+=s.br_poena;
			br_asistencija+=s.br_asistencija;
			br_skokova+=s.br_skokova;
			br_utakmica++;
		}
	}
	
	bool IsPlayer(int broj)
	{
		if(id==broj) return true;
		return false;
	}
	
	
	bool operator>(Kosarkas k)
	{
		double mojmvp=((br_poena*2+br_asistencija*3+br_skokova*3)/br_utakmica);
		double drugimvp=((k.br_poena*2+k.br_asistencija*3+k.br_skokova*3)/k.br_utakmica);
	
		if(mojmvp>drugimvp) return true;
		else return false;
	}
	
};


int main(int argc, char* argv[]) 
{
  mpi::environment env(argc, argv);
  mpi::communicator world;
  
  
  vector<Kosarkas> kosarkasi;
  vector<Stat> statistika;
  int i,j;
  int n;
  int koliko;
  int kolikoS;
	if(world.rank()==0)
	{
		FILE *f = fopen("dat.txt","r");
		fscanf(f,"%d",&n);
		
		int id;
		char ime[10];
	
		
		for (i = 0; i < n; i++)
		{
			fscanf(f,"%d%s",&id,ime);
			Kosarkas k(id,ime);			
			kosarkasi.push_back(k);
		}
		
		fscanf(f,"%d",&kolikoS);
		
		
		int brp,bras,brskok;
		for (i = 0; i < kolikoS; i++)
		{
			fscanf(f,"%d%d%d%d",&id,&brp,&bras,&brskok);
			Stat st(id,brp,brskok,bras);
			statistika.push_back(st);
		}
		//Stat(int _id, int _br_poena, int _br_skokova, int _br_asistencija)
		
		koliko = n/world.size();
		fclose(f);
	}


	broadcast(world,koliko,0);
	//podelim svima statistiku da bi setovali svoje podatke
	broadcast(world,statistika,0);
	
	Kosarkas mojiKosarkasi[koliko];
	
	//printf("koliko: %d\n",koliko);

	
	
	scatter(world,kosarkasi,mojiKosarkasi,koliko,0);
		//podelim  jednak br igraca svima
	
	//sad svi da setuju statistiku
	
	for (i = 0; i <koliko ; i++)
	{
		for (j = 0; j <statistika.size(); j++)
		{
			mojiKosarkasi[i].AddStat(statistika[j]); 
		}
		
	}
	//sad su svima setovane statistike;
	
	/*for (int i = 0; i < koliko; i++)
	{
		printf("%s\t%d  poeni:%d\n",mojiKosarkasi[i].ime,mojiKosarkasi[i].id,mojiKosarkasi[i].br_poena);	
	}
	
	*/
	
	//sad sort
	for (i = 0; i < koliko-1; i++)
	{
		for (j = i+1; j <koliko ; j++)
		{
			if(mojiKosarkasi[i]>mojiKosarkasi[j]==false)
			{
				Kosarkas temp = mojiKosarkasi[i];
				mojiKosarkasi[i]=mojiKosarkasi[j];
				mojiKosarkasi[j]=temp;
			}			
		}
		
	}
	
	//na prvom mestu mojiKosarkasi je najbolji u tom procesu
	
	Kosarkas konacno[world.size()]; //ovde cu da smestim najbolje
	//posaljem glavnom procesu 
	
	if(world.rank()!=0)
	{
		
		world.send(0,10,mojiKosarkasi[0]); 
	}
	else
	{	
		konacno[0]=mojiKosarkasi[0];
		for (i = 1; i < world.size(); i++)
		{
			world.recv(i,10,konacno[i]);
		}
		
	}
		
	
	if(world.rank()==0)
	{
		
		for (i = 0; i < world.size()-1; i++)
		{
			for (j = i+1; j <world.size() ; j++)
			{
				if(konacno[i]>konacno[j]==false)
				{
					Kosarkas temp = konacno[i];
					konacno[i]=konacno[j];
					konacno[j]=temp;
				}			
			}
		
		}
		 
		printf("Najbolji je:%s\t sa id-em:%d \n",konacno[0].ime,konacno[0].id);
	}

  return 0;
}
